% Script file: ErroreCS.m
%
% Analisi dell'errore per l'equazione del calore 
%            u_t = u_xx,  x in [0,L] 
% risolta con il metodo spettrale
% Lo script utilizza la funzione un1=CSPasso(un,dt,dx)
% Il dato iniziale e' 
%               u(x,0)=sin(pi*x/L)
% con condizioni al bordo di Dirichlet u(0,t)=0, u(L,t)=0.
% Il problema ammette la soluzione esatta
%               u(x,t)=exp(-t*pi^2/L^2)*sin(pi*x/L)
%

close all;
clear; clc;
L = 1;
n0=5;
N0 = 16 ;
Ne = 200 ;
xe=linspace(0,L,Ne);
dx0 = L/(N0-1);
t = n0*dx0 ;
kmax = 4 ;
passo = 8 ;
n = n0 ;
N = N0 ;
E=zeros(kmax,8*(N0-1)+1);

% Ripete kmax volte i calcoli con numero di punti crescente 

for k=1:kmax
   dx = L/(N-1);
   dt = dx;
   x=linspace(0,L,N);
   u=sin(pi*x/L);
   ue=sin(pi*xe/L);
   u=u';
   plot(x,u,'o',xe,ue,'r')
   xlabel('x')
   ylabel('u(x,t)')
   s=sprintf('Distribuzione iniziale per N=%1.0f:',N); 
   title(s);
   pause(1)
   tc = 0 ;
   
   % Calcola l'evoluzione al tempo t
   
   for h=0:n
      tc = tc+dt;
      u=CIPasso(u,dt,dx);
      ue=exp(-tc*pi^2/L^2)*sin(pi*xe/L);
      plot(x,u,'o',xe,ue,'r')
      s=sprintf('Evoluzione al tempo t=%f per N=%1.0f',tc,N);
      title(s);
      xlabel('x');
      ylabel('u(x,t)');
      axis([0 1 0 1]);
      drawnow
   end
   ue=exp(-tc*pi^2/L^2)*sin(pi*x/L);
   E(k,1:passo:8*(N0-1)+1)=abs(ue-u)/ue;
   n=n*2;
   N=2*N-1;
   passo = passo/2;
end

% Realizza il grafico dell'errore

figure
passo=8;
N=N0;
hold on
for k=1:kmax
   x=linspace(0,L,N);
   Errore(k)=max(E(k,1:passo:8*(N0-1)+1));
   plot(x,E(k,1:passo:8*(N0-1)+1),'-o');
   passo = passo /2;
   N=2*N-1;
   s=sprintf('Errore al tempo finale t=%f',tc);
   title(s);
   xlabel('x');
   ylabel('E(x,t)');
end
hold off

% Tabella con errore massimo e ordine di convergenza

disp('EQUAZIONE DEL CALORE: metodo implicito')
disp(' --------------------------------'); 
disp(sprintf('|N \t\t| Errore\t| Ordine\t|')); 
disp(' --------------------------------'); 
N=N0;
for k=1:kmax-1
   s=sprintf('|%1.0f\t\t| %3.2e\t|\t\t\t|',N,Errore(k));    
   disp(s);
   ordine(k)= log2(Errore(k)/Errore(k+1));
   s=sprintf('|\t\t|\t\t\t|\t%3.2f\t|',ordine(k));
   disp(s);
   N=2*N-1;
end
s=sprintf('|%1.0f\t| %3.2e\t|\t\t\t|',N,Errore(kmax));
disp(s);